@@ -2,7 +2,7 @@
 #
 # The fisx library for X-Ray Fluorescence
 #
-# Copyright (c) 2014-2020 European Synchrotron Radiation Facility
+# Copyright (c) 2014-2023 European Synchrotron Radiation Facility
 #
 # This file is part of the fisx X-ray developed by V.A. Sole
 #
@@ -83,17 +83,14 @@ Elements::Elements(std::string epdl97Directory)
 Elements::Elements(std::string directoryName, short pymca)
 {
     std::string BINDING_ENERGIES="EADL97_BindingEnergies.dat";
-    std::string joinSymbol;
     std::string bindingEnergies;
     std::string crossSections;
-
-#ifdef _WIN32
-    joinSymbol = "\\";
-#elif _WIN64
-    joinSymbol = "\\";
+#if defined(_WIN32) || defined(_WIN64) || defined(__MINGW32__) || defined(__CYGWIN32__)
+    std::string joinSymbol = "\\";
 #else
-    joinSymbol = "//";
-#endif
+    std::string joinSymbol = "//";
+#endif // __WIN32
+
     // check if directoryName already ends with the joinSymbol
     if (directoryName.substr(directoryName.size() - 1, 1) == joinSymbol)
     {
@@ -313,7 +310,7 @@ void Elements::addElement(const Element & element)
     }
     else
     {
-        this->elementDict[name] = this->elementList.size();
+        this->elementDict[name] = (int) this->elementList.size();
         this->elementList.push_back(element);
     }
 }
@@ -1050,8 +1047,9 @@ std::map<std::string, std::vector<double> > Elements::getMassAttenuationCoeffici
 }
 
 std::map<std::string, double> Elements::getMassAttenuationCoefficients(\
-                                                std::map<std::string, double> inputFormulaDict,\
-                                                double inputEnergy) const
+                                                const std::map<std::string, double> & inputFormulaDict,\
+                                                const double & inputEnergy, \
+                                                const int & isComposition) const
 {
     std::vector<double> energy;
     std::map<std::string, std::vector<double> >tmpResult;
@@ -1059,7 +1057,7 @@ std::map<std::string, double> Elements::getMassAttenuationCoefficients(\
 
     energy.resize(1);
     energy[0] = inputEnergy;
-    tmpResult = this->getMassAttenuationCoefficients(inputFormulaDict, energy);
+    tmpResult = this->getMassAttenuationCoefficients(inputFormulaDict, energy, isComposition);
 
     result["energy"] = tmpResult["energy"][0];
     result["coherent"] = tmpResult["coherent"][0];
@@ -1072,8 +1070,9 @@ std::map<std::string, double> Elements::getMassAttenuationCoefficients(\
 }
 
 std::map<std::string, std::vector<double> > Elements::getMassAttenuationCoefficients(\
-                                                std::map<std::string, double> inputFormulaDict,\
-                                                                std::vector<double> energy) const
+                                                const std::map<std::string, double> & inputFormulaDict,\
+                                                const std::vector<double> & energy, \
+                                                const int & isComposition) const
 {
     std::string element, msg, name;
     double total, massFraction;
@@ -1086,42 +1085,51 @@ std::map<std::string, std::vector<double> > Elements::getMassAttenuationCoeffici
     std::map<std::string, double>::iterator it;
     std::map<std::string , int>::const_iterator mapIterator;
 
-    total = 0.0;
-    for (c_it = inputFormulaDict.begin(); c_it != inputFormulaDict.end(); ++c_it)
+    if (isComposition)
     {
-        massFraction = c_it->second;
-        if (massFraction < 0.0)
+        // no need to parse anything
+        elementsDict = inputFormulaDict;
+        total = 1.0;
+    }
+    else
+    {
+        total = 0.0;
+        for (c_it = inputFormulaDict.begin(); c_it != inputFormulaDict.end(); ++c_it)
         {
-            msg = "Name " + c_it->first + " has a negative mass fraction!!!";
-            throw std::invalid_argument(msg);
-        }
-        // we may have received formulas ...
-        name = c_it->first;
-        //composition = this->getCompositionFromFormula(name);
-        composition = this->getComposition(name);
-        if (composition.size() < 1)
-        {
-            msg = "Name " + c_it->first + " not understood";
-            std::cout << msg << std::endl;
-            throw std::invalid_argument(msg);
-        }
-        for(it = composition.begin(); it != composition.end(); ++it)
-        {
-            composition[it->first] *= massFraction;
-            if (elementsDict.find(it->first) == elementsDict.end())
+            massFraction = c_it->second;
+            if (massFraction < 0.0)
             {
-                elementsDict[it->first] = 0.0;
+                msg = "Name " + c_it->first + " has a negative mass fraction!!!";
+                throw std::invalid_argument(msg);
             }
-            elementsDict[it->first] += composition[it->first];
+            // we may have received formulas ...
+            name = c_it->first;
+            //composition = this->getCompositionFromFormula(name);
+            composition = this->getComposition(name);
+            if (composition.size() < 1)
+            {
+                msg = "Name " + c_it->first + " not understood";
+                std::cout << msg << std::endl;
+                throw std::invalid_argument(msg);
+            }
+            for(it = composition.begin(); it != composition.end(); ++it)
+            {
+                composition[it->first] *= massFraction;
+                if (elementsDict.find(it->first) == elementsDict.end())
+                {
+                    elementsDict[it->first] = 0.0;
+                }
+                elementsDict[it->first] += composition[it->first];
+            }
+            total += massFraction;
+        }
+        if (total <= 0.0)
+        {
+            msg = "Sum of mass fractions is less or equal to 0";
+            throw std::invalid_argument(msg);
         }
-        total += massFraction;
     }
 
-    if (total <= 0.0)
-    {
-        msg = "Sum of mass fractions is less or equal to 0";
-        throw std::invalid_argument(msg);
-    }
 
     result["energy"].resize(energy.size());
     result["coherent"].resize(energy.size());
@@ -1383,6 +1391,44 @@ std::map<std::string, double> Elements::getComposition(const std::string & name)
     return result;
 }
 
+std::map<std::string, double> Elements::getComposition(const std::map<std::string, double> & inputComposition, \
+                                                       const std::vector<Material> & additionalMaterials) const
+{
+    std::map<std::string, double>::const_iterator c_it, c_it2;
+    std::map<std::string, double> tmpComposition;
+    std::map<std::string, double> result;
+    double total;
+
+    for (c_it = inputComposition.begin(); c_it != inputComposition.end(); ++c_it)
+    {
+        tmpComposition = this->getComposition(c_it->first, additionalMaterials);
+        if (tmpComposition.size() < 1)
+        {
+            return tmpComposition;
+        }
+        for (c_it2 = tmpComposition.begin(); c_it2 != tmpComposition.end(); ++c_it2)
+        {
+            if (result.find(c_it2->first) == result.end())
+            {
+                result[c_it2->first] = 0.0;
+            }
+            result[c_it2->first] += c_it2->second * c_it->second;
+        }
+    }
+
+    // make sure that everything is normalized (not warranted from input composition)
+    total = 0;
+    for (c_it = result.begin(); c_it != result.end(); ++c_it)
+    {
+        total += c_it->second;
+    }
+    for (c_it = result.begin(); c_it != result.end(); ++c_it)
+    {
+        result[c_it->first] /= total;
+    }
+    return result;
+}
+
 std::map<std::string, double> Elements::getComposition(const std::string & name, \
                                                  const std::vector<Material> & additionalMaterials) const
 {
@@ -1526,6 +1572,43 @@ std::map<std::string, std::map<std::string, double> > Elements::getExcitationFac
     return this->getExcitationFactors(element, energies, weights)[0];
 }
 
+std::map<std::string, std::vector <double> > Elements::getInitialPhotoelectricVacancyDistribution(\
+                                                const std::string & elementName,
+                                                const std::vector<double> & energies) const
+{
+    const Element & elementObject = this->getElement(elementName);
+    return elementObject.getInitialPhotoelectricVacancyDistribution(energies);
+}
+
+
+std::map<std::string, double> Elements::getInitialPhotoelectricVacancyDistribution(\
+                                                                         const std::string & elementName,
+                                                                         const double & energy) const
+{
+    const Element & elementObject = this->getElement(elementName);
+    return elementObject.getInitialPhotoelectricVacancyDistribution(energy);
+}
+
+std::map<std::string, double> Elements::getCascadeModifiedVacancyDistribution(const std::string & elementName,
+                                                                    const std::map<std::string, \
+                                                                    double> & distribution) const
+{
+    const Element & elementObject = this->getElement(elementName);
+    return elementObject.getCascadeModifiedVacancyDistribution(distribution);
+
+}
+
+std::map<std::string, std::map<std::string, double> >\
+Elements::getXRayLinesFromVacancyDistribution(const std::string & elementName,
+                                              const std::map<std::string, double> & distribution, \
+                                              const int & cascade, \
+                                              const int & useFluorescenceYield) const
+{
+    const Element & elementObject = this->getElement(elementName);
+    return elementObject.getXRayLinesFromVacancyDistribution(distribution, cascade, useFluorescenceYield);
+
+}
+
 std::map<std::string, double> Elements::parseFormula(const std::string & formula) const
 {
     std::map<std::string, double> composition;
@@ -1533,9 +1616,9 @@ std::map<std::string, double> Elements::parseFormula(const std::string & formula
     std::map<std::string, double>::iterator it;
     std::string::size_type i, p1, p2, length;
     std::string::const_iterator c_it;
-    std::vector<int> openParenthesis;
-    std::vector<int> closeParenthesis;
-    std::vector<int>::size_type nParenthesis;
+    std::vector<std::string::size_type> openParenthesis;
+    std::vector<std::string::size_type> closeParenthesis;
+    std::vector<std::string::size_type>::size_type nParenthesis;
     std::string newFormula;
     bool parsingKey;
     std::string lastKey;
@@ -2028,7 +2111,6 @@ const std::vector<Material>::size_type Elements::getMaterialIndexFromName(const
 {
     std::vector<Material>::size_type i;
 
-    this->materialList.begin();
     for (i = 0; i < this->materialList.size(); i++)
     {
         if(this->materialList[i].getName() == name)
