@@ -2,7 +2,7 @@
 #
 # The fisx library for X-Ray Fluorescence
 #
-# Copyright (c) 2014-2020 European Synchrotron Radiation Facility
+# Copyright (c) 2014-2023 European Synchrotron Radiation Facility
 #
 # This file is part of the fisx X-ray developed by V.A. Sole
 #
@@ -80,7 +80,7 @@ public:
     Funtion to set a single energy excitation beam
     \param energy Energy of the incoming beam
     */
-    void setBeam(const double & energy, const double & divergency = 0.0);
+    void setSingleEnergyBeam(const double & energy, const double & divergency = 0.0);
 
 
     /*!
@@ -163,6 +163,11 @@ public:
     void setCollimators();
     void addCollimator();
 
+    /*!
+    For debugging purposes
+    */
+    void printConfiguration() const;
+
     /*!
     Get the current configuration
     */
@@ -208,9 +213,9 @@ public:
                 const Elements & elementsLibrary, const int & sampleLayerIndex = 0, \
                 const std::string & lineFamily = "", const int & secondary = 0, \
                 const int & useGeometricEfficiency = 1, const int & useMassFractions = 0, \
-                const double & secondaryCalculationLimit = 0.0);
+                const double & secondaryCalculationLimit = 0.0, \
+                const Beam & overwritingBeam = Beam());
     /*!
-    Basis method called by all the other convenience methods.
     \param elementFamilyLayer - Vector of strings. Each string represents the information we are interested on.\n
     "Cr"     - We want the information for Cr, for all line families and sample layers\n
     "Cr K"   - We want the information for Cr, for the family of K-shell emission lines, in all layers.\n
@@ -239,11 +244,45 @@ public:
     */
     std::map<std::string, std::map<int, std::map<std::string, std::map<std::string, double> > > > \
                 getMultilayerFluorescence(const std::vector<std::string> & elementFamilyLayer, \
-                const Elements & elementsLibrary, const int & secondary = 0, \
+                const Elements & elementsLibrary, \
+                const int & secondary = 0, \
                 const int & useGeometricEfficiency = 1, \
                 const int & useMassFractions = 0, \
-                const double & secondaryCalculationLimit = 0.0);
+                const double & secondaryCalculationLimit = 0.0,
+                const Beam & overwritingBeam = Beam());
 
+    /*!
+    Basis method called by all the other convenience methods.
+    \param elementList - Vector of strings. Each string represents one element.\n
+    \param layerList - Vector of integers representing the sample layers where the calculation for the elements in
+    the elements list takes place. Either a length of 1 or of the same length as the element list.
+    \param familyList - Vector of strings. Each string represents one peak family.\n
+    The vector has to be of the same size as the elements list.
+    \param elementsLibrary - Instance of library to be used for all the Physical constants
+    \param secondary - Flag to indicate different levels of secondary excitation to be considered.\n
+                0 Means not considered\n
+                1 Consider secondary excitation\n
+                2 Consider tertiary excitation\n
+    \param useGeometricEfficiency - Take into account solid angle or not. Default is 1 (yes)
+
+    \param useMassFractions - If 0 (default) the output corresponds to the requested information if the mass
+    fraction of the element would be one on each calculated sample layer. To get the actual signal, one
+    has to multiply the rates by the actual mass fraction of the element on each sample layer.
+                       If set to 1, the rate will be already corrected by the actual mass fraction.
+
+    \param overwritingBeam - Optional parameter to use a different beam than the one in the configuration.
+
+    \return Return a complete output of the form:\n
+    [Element Family][Layer][line]["energy"] - Energy in keV of the emission line\n
+    [Element Family][Layer][line]["primary"] - Primary rate prior to correct for detection efficiency\n
+    [Element Family][Layer][line]["secondary"] - Secondary rate prior to correct for detection efficiency\n
+    [Element Family][Layer][line]["rate"] - Overall rate\n
+    [Element Family][Layer][line]["efficiency"] - Detection efficiency\n
+    [Element Family][Layer][line][element line layer] - Secondary rate (prior to correct for detection efficiency)
+    due to the fluorescence from the given element, line and layer index composing the map key.\n
+    [Element Family][Layer][line]["massFraction"] - Mass fraction of the element in the considered layer
+
+    */
     std::map<std::string, std::map<int, std::map<std::string, std::map<std::string, double> > > > \
                 getMultilayerFluorescence(const std::vector<std::string> & elementList,
                                           const Elements & elementsLibrary, \
@@ -252,7 +291,8 @@ public:
                                           const int & secondary = 0, \
                                           const int & useGeometricEfficiency = 1, \
                                           const int & useMassFractions = 0, \
-                                          const double & secondaryCalculationLimit = 0.0);
+                                          const double & secondaryCalculationLimit = 0.0,
+                                          const Beam & overwritingBeam = Beam()) const;
 
 
     double getEnergyThreshold(const std::string & elementName, const std::string & family, \
@@ -298,7 +338,73 @@ public:
     const double & getAlphaOut() const {return this->configuration.getAlphaOut();};
     const double & getScatteringAngle() const {return this->configuration.getScatteringAngle();};
     const int & getReferenceLayer() const {return this->configuration.getReferenceLayer();};
+    const std::vector<Material> & getMaterials() const {return this->configuration.getMaterials();};
 
+    /*
+    Replacement of layer methods accounting for materials defined in the configuration.
+    Only the XRF instance knows about all the materials defined. That is not the case for the elements library.
+    */
+
+    /*!
+    Get the layer composition either from internal material or from name.
+    In the later case the elements instance is not used.
+    */
+    std::map<std::string, double> getLayerComposition(const Layer & layer, const Elements & elements) const;
+
+    /*!
+    Get the layer mass attenuation coefficients transmission at the given energy using the elements library
+    supplied but accounting for the materials defined in the configuration.
+    If the layer material is not defined or cannot be handled, it will throw the
+    relevant error.
+    */
+    std::map<std::string, double> getLayerMassAttenuationCoefficients(const Layer & layer,
+                                                                      const double & energy,
+                                                                      const Elements & elements,
+                                                const std::map<std::string, double> & layerComposition = std::map<std::string, double>()) const;
+
+
+    /*!
+    Get the layer mass attenuation coefficients transmission at the given energies using the elements
+    library supplied but accounting for the materials defined in the configuration.
+    If a material is not defined or cannot be handled, it will throw the
+    relevant error.
+    */
+    std::map<std::string, std::vector<double> > getLayerMassAttenuationCoefficients( \
+                                                const Layer & layer,
+                                                const std::vector<double> & energies,
+                                                const Elements & elements,
+                                                const std::map<std::string, double> & layerComposition = std::map<std::string, double>()) const;
+    /*!
+    Get the layer transmissions at the given energy using the elements library
+    supplied but accounting for the materials defined in the configuration.
+    If a material is not defined or cannot be handled, it will throw the
+    relevant error.
+    */
+    double getLayerTransmission(const Layer & layer, const double & energy, const Elements & elements, const double & angle = 90.0, \
+                                                const std::map<std::string, double> & layerComposition = std::map<std::string, double>()) const;
+
+    /*!
+    Get the layer transmissions at the given energies using the elements library
+    supplied but accounting for the materials defined in the configuration.
+    If a material is not defined or cannot be handled, it will throw the
+    relevant error.
+    */
+    std::vector<double> getLayerTransmission(const Layer & layer,
+                                             const std::vector<double> & energy,
+                                             const Elements & elements,
+                                             const double & angle = 90.0,
+                                             const std::map<std::string, double> & layerComposition = std::map<std::string, double>()) const;
+
+    /*!
+    Get the emitted peak families from a layer given an energy and a reference to an elements library.
+    It uses the supplied Elements library accounting for the materials defined in the configuration.
+    It returns an ordered vector of pairs.
+    The first element is the peak family ("Si K", "Pb L1", ...) and the second the binding energy.
+    */
+    std::vector<std::pair<std::string, double> > getLayerPeakFamilies(const Layer & layer,
+                                             const double & energy,
+                                             const Elements & elements,
+                                             const std::map<std::string, double> & layerComposition = std::map<std::string, double>()) const;
 private:
     /*!
     Reference to elements library to be used for calculations
